
// --------------------------------------------------------------------------------------
//  values 
//

var pkg = require('./package.json');

var gulp = require('gulp');

var merge = require('event-stream').merge;

var browserSync = require('browser-sync');
var reload = browserSync.reload;

var babel = require('gulp-babel');
var eslint = require('gulp-eslint');
var concat = require('gulp-concat');
var uglify = require('gulp-uglify');

var base64 = require('gulp-base64');

var del = require('del');
var sprite = require('gulp.spritesmith');

var sass = require('gulp-sass');
var minify = require('gulp-clean-css');
var plumber = require('gulp-plumber');

var imagemin = require('gulp-imagemin');
var pngquant = require('imagemin-pngquant');

var path = {
	js: 'js',
	css: 'css',
	html: 'html/**/*'
}

var replace = require('replace');

var fs = require('fs');
var aPath = require('path');

var getFolders = function(dir) {

	return fs.readdirSync(dir).filter( function(file) {

			return fs.statSync(aPath.join(dir, file)).isDirectory();
	});
}

var runSequence = require('run-sequence');
var pngfiles;






// --------------------------------------------------------------------------------------
//  replace label of project 
//

var directoryName = __dirname.slice(__dirname.lastIndexOf('/') + 1);

if (pkg.name !== directoryName) {

    var oldName = pkg.name;

	var replaceFileName = [
		'html/conf.html',
		'html/module/meta/jsBottom.html',
		'package.json',
	];

	var jsmodFileName = fs.readdirSync('jsmod/src/');

	for (var i = 0; i < jsmodFileName.length; i++){
		replaceFileName.push('jsmod/src/' + jsmodFileName[i]);
	}

	for (var i = 0; i < replaceFileName.length; i++){
		replace({
			regex: pkg.name,
			replacement: directoryName,
			paths: [replaceFileName[i]],
			recursive: true,
			silent: true,
		});
	}

    fs.rename('sass/' + oldName+ '.scss', 'sass/' + directoryName + '.scss');

    del([
        'css/' + oldName + '.css',
        'js/' + oldName + '.js'
    ]);

    pkg.name = directoryName;

}



// --------------------------------------------------------------------------------------
//  task: js check and concat
//
gulp.task('eslint', function() {

	return gulp.src([
            'jsmod/src/**/*.js'
		]).pipe(eslint())
		.pipe(eslint.format())
		.pipe(eslint.failOnError());
});

gulp.task('concat', ['eslint'], function() {

	gulp.src([
			//'node_modules/jquery/dist/jquery.js',
			//'node_modules/zepto/zepto.min.js',
            'jsmod/src/_init.js',
            'jsmod/src/lib/**/*.js',
            'jsmod/src/fnc/**/*.js'
		])
    	.pipe(concat(pkg.name + '.js'))
    	.pipe(gulp.dest(path.js))
});



// --------------------------------------------------------------------------------------
//  task: babel compile, js check and concat
//
gulp.task('babel_init', function() {

    del('jsmod/make/**/*.js');

    return gulp.src([
            'jsmod/src/_init.js',
        ])
        .pipe(babel({
            presets: ['es2015']
        }))
        .pipe(gulp.dest('jsmod/make/babel'));
});

gulp.task('babel_lib', ['babel_init'], function() {

    return gulp.src([
            'jsmod/src/lib/**/*.js',
        ])
        .pipe(babel({
            presets: ['es2015']
        }))
        .pipe(gulp.dest('jsmod/make/babel/lib'));
});

gulp.task('babel_fnc', ['babel_lib'], function() {

    return gulp.src([
            'jsmod/src/fnc/**/*.js'
        ])
        .pipe(babel({
            presets: ['es2015']
        }))
        .pipe(gulp.dest('jsmod/make/babel/fnc'));
});

gulp.task('babel_eslint', ['babel_fnc'], function() {

	return gulp.src([
			'jsmod/make/babel/**/*.js',
		]).pipe(eslint())
		.pipe(eslint.format())
		.pipe(eslint.failOnError());
});

gulp.task('babel_concat', ['babel_eslint'], function() {

	return gulp.src([
			//'node_modules/jquery/dist/jquery.js',
			//'node_modules/zepto/zepto.min.js',
			'jsmod/make/babel/_init.js',
			'jsmod/make/babel/lib/**/*.js',
			'jsmod/make/babel/fnc/**/*.js',
		])
    	.pipe(concat(pkg.name + '.js'))
    	.pipe(gulp.dest(path.js))
});



// --------------------------------------------------------------------------------------
//  task: base64 
//

// The folder 'base64' include a few png files ?
fs.readdir('base64', function(err, list) {
	list.forEach( function(file) {
		if (aPath.extname(file) === '.png') {
			pngfiles = true;
		}
	})
})

gulp.task('base64_map', function(pngfiles) {

	if (pngfiles) {

		var base64Options = {
			imgName: 'base64.png',
			imgPath: 'base64/make/base64.png',
			cssName: '_map.scss',
			cssFormat: 'scss',
			cssOpts: {
				functions: false 
			},
			cssSpritesheetName: 'base64',
			algorithm: 'binary-tree',
			padding: 4
		}

		var base64Data = gulp.src('base64/*.png').pipe(sprite(base64Options));

		base64Data.img.pipe(gulp.dest('base64/make'));
		return base64Data.css.pipe(gulp.dest('sass/elements/make/base64'));	
	}
});

gulp.task('base64_url', ['base64_map'], function() {

	if (pngfiles) {

		return gulp.src('sass/elements/lib/_base64.scss')
			.pipe(base64())
			.pipe(concat('_url.scss'))
			.pipe(gulp.dest('sass/elements/make/base64'));
	}
});



// --------------------------------------------------------------------------------------
//  task: sprite
//

gulp.task('reset_spr', function() {
    del('img/make/*.png');
});

gulp.task('sprites', ['reset_spr'], function() {

    var rdm = (new Date()).getTime();

	var folders = getFolders('sprite/');

	if (folders[0] !== undefined) {

		return merge(

			folders.map( function(folder) {

				spriteOptions = {
                    imgName: 'spr-' + folder + '-' + rdm + '.png',
                    imgPath: 'img/make/spr-' + folder + '-' + rdm + '.png',
					cssName: '_spr_' + folder + '.scss',
					cssFormat: 'scss',
					cssOpts: {
						functions: false 
					},
					cssVarMap: function(sprite) {
						sprite.name = folder + '-' + sprite.name;
					},
					cssSpritesheetName: 'spr-' + folders.indexOf(folder),
					algorithm: 'binary-tree',
					padding: 4
				}

				spriteData = gulp.src('sprite/' + folder + '/*.png').pipe(sprite(spriteOptions));

				spriteData.img.pipe(gulp.dest('img/make'));
				return spriteData.css.pipe(gulp.dest('sass/elements/make/spr'));	
			})
		);
	}
});



// --------------------------------------------------------------------------------------
//  task: utility scss concat 
//

gulp.task('scss_concat', function() {

    return gulp.src([
            'sass/elements/make/spr/*.scss',
            'sass/elements/make/base64/*.scss',
            'sass/elements/lib/_utility.scss'
        ])
        .pipe(concat('_concat.scss'))
        .pipe(gulp.dest('sass/elements/make'))
});



// --------------------------------------------------------------------------------------
//  task: sass compile 
//

gulp.task('sass', ['scss_concat'], function() {

    return gulp.src([
			'sass/**/*.scss',
			'!sass/elements/make/base64/*.scss',
			'!sass/elements/make/spr/*.scss',
			'!sass/elements/lib/_base64.scss',
			'!sass/elements/lib/_utility.scss'
		])
        .pipe(plumber())
        .pipe(sass({outputStyle: 'expanded'}))
        .pipe(gulp.dest(path.css))
});



// --------------------------------------------------------------------------------------
//  task: optimize image 
//

gulp.task('opt_image', function() {

	return gulp.src('img/**/*').pipe(imagemin({
		use: [
			pngquant({
				quality: 60 - 80,
				speed: 1
			})
		]
	})).pipe(gulp.dest('img/'));
});



// --------------------------------------------------------------------------------------
//  task: watch and live reload 
//

gulp.task('watch', function() {

    gulp.watch('jsmod/src/**/*', ['concat', reload]);
    //gulp.watch('jsmod/src/**/*', ['babel_concat', reload]);
    gulp.watch('sass/**/*', ['sass', reload]);
    gulp.watch('html/**/*', reload);
});

gulp.task('server', function() {

	browserSync({
		proxy: 'http://localhost/' + pkg.name 
	});
});



// --------------------------------------------------------------------------------------
//  task: uglify js
//

gulp.task('uglify', function() {

	gulp.src('js/*')
		.pipe(uglify())
		.pipe(gulp.dest(path.js))
});



// --------------------------------------------------------------------------------------
//  task: minify css
//

gulp.task('minify', function() {

    gulp.src('css/*')
        .pipe(minify())
        .pipe(gulp.dest(path.css))
});



// --------------------------------------------------------------------------------------
//  task: run
//

// Default
gulp.task('default', function() {
    runSequence('concat', 'sass');
});

// Default with babel
//gulp.task('default', function() {
//    runSequence('babel_concat', 'sass');
//});

// Make sprite & base64 images
gulp.task('i', function() {
    runSequence('base64_url', 'sprites', 'sass', 'opt_image');
});
// Watch and Live reload
gulp.task('w', function() {
    runSequence('watch', 'server');
});
// Release version
gulp.task('r', function() {
    runSequence('uglify', 'minify');
});




